@@ -1,12 +1,9 @@
-import json
-import os
-
 import requests
 from requests.adapters import HTTPAdapter
 from requests.packages.urllib3.util.retry import Retry
 from requests.exceptions import ConnectTimeout, ReadTimeout
 
-DEFAULT_URL = "https://api.freedomrobotics.ai"
+from freedomrobotics_api.credentials import credentials_exist, get_credentials, URLS, DEFAULT_URL
 
 default_api = None
 
@@ -26,6 +23,38 @@ def api_call(*args, **kwargs):
     return default_api.call(*args, **kwargs)
 
 
+def get_url(url=None, env=None, credentials=None):
+    """Selects the correct url looking at the available ones.
+    The priority order is:
+    1. explicit url
+    2. env url
+    3. credentials url
+    4. default url
+    """
+    if url is not None:
+        return url
+    if env is not None and env in URLS:
+        return URLS[env]
+    if credentials is not None and credentials.url is not None:
+        return credentials.url
+    return DEFAULT_URL
+
+
+def login(username, password, url=None, env=None):
+    url = get_url(url=url, env=env)
+    response = APIHandler.simple_call(
+        url,
+        "PUT",
+        "/users/{}/login".format(username),
+        data={
+            "password": password
+        }
+    )
+    token = response.get("token")
+    secret = response.get("secret")
+    return url, token, secret
+
+
 class APIHandler(object):
 
     def __init__(self,
@@ -83,36 +112,21 @@ class APIHandler(object):
         return session
 
     @classmethod
-    def get_instance(cls, url=None, token=None, secret=None, username=None, password=None, **kwargs):
+    def get_instance(cls, url=None, token=None, secret=None, username=None, password=None, env=None, **kwargs):
         if token is not None and secret is not None:
             token = token
             secret = secret
-            if url is None:
-                url = DEFAULT_URL
-
+            url = get_url(url, env)
         # if class is initialized with a username and password, use it
         elif username is not None and password is not None:
-            if url is None:
-                url = DEFAULT_URL
-            response = cls.simple_call(
-                url,
-                "PUT",
-                "/users/{}/login".format(username),
-                data={
-                    "password": password
-                }
-            )
-            token = response.get("token")
-            secret = response.get("secret")
+            url, token, secret = login(token, secret, url=url, env=env)
 
         # if there is a saved freedom credentials file, use it
-        elif os.path.exists(os.path.expanduser("~/.freedom_credentials")):
-            with open(os.path.expanduser("~/.freedom_credentials")) as f:
-                credentials = json.load(f)
-                if url is None:
-                    url = credentials.get("url", DEFAULT_URL)
-                token = credentials.get("token")
-                secret = credentials.get("secret")
+        elif credentials_exist():
+            credentials = get_credentials()
+            url = get_url(url, env, credentials=credentials)
+            token = credentials.token
+            secret = credentials.secret
         else:
             raise Exception("Required: either token/secret, username/password, or saved ~/.freedom_credentials file")
 
